/*
 * LEDcube.cpp - Arduino library for LEDcube V.2 kit
 *
 *      This work is licensed under the Creative Commons BY SA License.Creative Commons v3.5
 *           Attribution and Share Alike.
 *  	see http://creativecommons.org/licenses/by-sa/3.0/ for details.
 *      Rolf van Widenfelt, Mitch Altman.
 *
 *
 *	TODO:
 *		- WaitUntilFrame() stuff is NYI!
 *
 *	revision history:
 *
 *	- apr 10, 2011 - mitch
 *		create TRUE and FALSE as bool types, and change GetButton() to bool
 *	- mar 1, 2011 - rolf
 *		create a "Begin()" method and set the timer1 regs in there.
 *		the application will need to call this from setup().
 *		anyway... TIMER1 now works! woohoo!!
 *		add GetButton() method.
 *		some code cleanup. (e.g. remove debug LED from ISR)
 *
 *	- feb 28, 2011 - rolf
 *		much debugging effort... (thanks for help from Anthony Roberts!)
 *		but, timer1 is still not behaving as configured.
 *		this suggests the arduino environment is setting it after my code below... argh!
 *
 *	- feb 27, 2011 - rolf
 *		configure TIMER1 for 1ms operation.  (fast PWM mode)
 *		still some bug regarding the actual interrupt rate (not nearly 1khz yet)
 *
 *	- feb 26, 2011 - rolf
 *		basic functionality mostly coded.
 *
 *	- feb 24, 2011 - rolf
 *		created.
 *
 */

extern "C" {
#include <inttypes.h>
#include <avr/io.h>				/* this takes care of definitions for our specific AVR */
#include <avr/interrupt.h>		/* for interrupts, ISR macro, etc. */
}

#include "LEDcube.h"
#include "ledcube_iodefs.h"		/* defines IO pins, and output_low(), output_high() macros */

/* note: the following global vars are defined outside of LEDcube class definition */

static uint8_t curcolor;
static uint8_t finterval;

static volatile uint8_t fb[_FB_SIZE];	/* 3x3x3 frame buffer - each byte holds a value from 0 to 0xF */


// used by ISR
volatile uint8_t _CurPlane;
volatile uint8_t _SwapRelease;	// flag (1 bit)
volatile uint8_t _SwapCounter;

// forward references
static void avrinit(void);
void start_timer1(void);


LEDcube::LEDcube(void)
{
	avrinit();		// configures port pins on the AVR

	curcolor = COLOR_ON;
	finterval = 1;
	Clear();

	// used in ISR
	_CurPlane = 0;

	//start_timer1();		// XXX we set this in the Begin method now
}


void LEDcube::Begin(void)
{
	start_timer1();
}


void LEDcube::SetColor(uint8_t c)
{
	curcolor = c;
}


uint8_t LEDcube::GetColor(void)
{
	return curcolor;
}


void LEDcube::Clear(void)
{
	for (uint8_t i = 0; i < _FB_SIZE; i++) {
		fb[i] = 0;
	}
}


void LEDcube::DrawPixel(uint8_t x, uint8_t y, uint8_t z)
{
	uint8_t indx;

	// clip to 3x3x3 cube
	if ((x < 3) && (y < 3) && (z < 3)) {
		indx = x + 3*y + 9*z;			// mults are fast, right??

		fb[indx] = curcolor;
	}
}


bool LEDcube::GetButton(uint8_t b)
{
	if (b != BUTTON_A) {	// there is only one button, BUTTON_A
		return FALSE;
	}

	// button pin is 0 when it is pressed, 1 otherwise

	return ((input_test(BUT1) == 0) ? TRUE : FALSE);	// XXX this does no debouncing
}



void LEDcube::SetFrameInterval(uint8_t f)
{
	if (f > 0) {
		finterval = f;
	}
}


void LEDcube::WaitUntilFrame(void)
{
	// XXX
}


/*
 *
 *	low level init needed for AVR.
 *
 */
static void avrinit(void)
{

	cli();	// needed??

	// note: these MUST be in sync with actual hardware!  (also see ledcube_iodefs.h)

	// note: DDR pins are set to "1" to be an output, "0" for input.

	//          76543210
	//PORTB = 0b00000001;		// initial: set to enable pullups on inputs: BUT1 pullup
	//DDRB  = 0b00100000;		// inputs: BUT1 (PB0); outputs: SCK (PB5); reserved (PB6, PB7)
	PORTB = 0x01;			// (see above)
	DDRB  = 0x30;			// (see above)

	//          76543210
	//PORTC = 0b00000000;		// initial: no pullups on inputs
	//DDRC  = 0b00111111;		// outputs: CC33-CC12 (PC0-PC5)
	PORTC = 0x00;		// (see above)
	DDRC  = 0x3F;		// (see above)

	//          76543210
	//PORTD = 0b00000000;		// initial: no pullups on inputs
	//DDRD  = 0b11111100;		// outputs: PP3-PP1 (PD2-PD4), CC31-CC11 (PD5-PD7); reserved (PD0/RxD,PD1/TxD)

	PORTD = 0x00;		// (see above)
	DDRD  = 0xFC;		// (see above)


	sei();					// enable interrupts (individual interrupts still need to be enabled)
}


//
//	set up timer1 in "fast PWM" mode 14 (see waveform generation, pg 137 of atmega168/328 doc - doc8271).
//
//	note: this causes TIMER1 to interrupt every 1ms! (TIMER1_OVF)
//	note: it's super important to call this AFTER arduino environment does it's good (and bad)
//		things to various TIMER settings.
//
void start_timer1(void)
{

	// make sure interrupts are off
	cli();

	// disable other timers?  not sure if this is needed XXX
	//TIMSK0 &= ~_BV(TOIE0);
	TIMSK1 &= ~_BV(TOIE1);
	TIMSK2 &= ~_BV(TOIE2);

	// initialize ICR1, which sets the "TOP" value for the counter to interrupt and start over
	// note: value of 125-1 ==> 1khz (assumes 8mhz clock, prescaled by 1/64)

	ICR1 = 125-1;
	OCR1A = 0;		// needed?? (no)

	//
	// timer1 config:
	// fast pwm (mode 14) - set WGM13,WGM12,WGM11 bits
	// prescaler clock/64 - set CS11,CS10 bits
	//
	TCCR1A = _BV(WGM11);
	TCCR1B = _BV(WGM13) | _BV(WGM12) | _BV(CS11) | _BV(CS10);

	TCCR1C = 0;	// not needed?

	TIMSK1 = _BV(TOIE1);		// enable timer1 overflow interrupt.. now were running!

	sei();
}


volatile uint8_t _Toggle = 0;		// debug only - cause LED1 to blink at 1/2 interrupt rate


ISR(TIMER1_OVF_vect)
{
	// there are 3 passes through this interrupt to draw the full cube

	uint8_t cbits, dbits;	// bits for PORTC, PORTD

#ifdef LEDCUBE_DEBUG
if (_Toggle == 0) {		// XXX for debug only!!
	output_high(LED1);
} else {
	output_low(LED1);
}
_Toggle = !_Toggle;
#endif

	//
	// for each pass, we display 1 plane, which is 9 (of 27) pixels.
	// these 9 pixels (LEDs) stay on until the next pass.
	//
	// for each "case", we set PORTC and PORTD bits.
	//	note: PORTD contains the plane enables (PP1-PP3) so we set it first
	//
	switch (_CurPlane) {
		case 0:
			//
			// pixels 0,1,2 go to IO pins D7,D6,D5
			//	also, add in 3 bits to turn off all the planes (PP1-PP3) by pulling them high
			//
			dbits = ((fb[0] & 0x8) << 4) |
					((fb[1] & 0x8) << 3) |
					((fb[2] & 0x8) << 2) |
					(0x7 << 2);
			PORTD = dbits;

			//
			// pixels 3-8 go to IO pins C5-C0
			//
			cbits = ((fb[3] & 0x8) << 2) |
					((fb[4] & 0x8) << 1) |
					((fb[5] & 0x8))      |
					((fb[6] & 0x8) >> 1) |
					((fb[7] & 0x8) >> 2) |
					((fb[8] & 0x8) >> 3);
			PORTC = cbits;

			output_low(PP1);	// finally, enable this plane!
			break;

		case 1:
			//
			// pixels 0,1,2 go to IO pins D7,D6,D5
			//	also, add in 3 bits to turn off all the planes (PP1-PP3) by pulling them high
			//
			dbits = ((fb[0+9] & 0x8) << 4) |
					((fb[1+9] & 0x8) << 3) |
					((fb[2+9] & 0x8) << 2) |
					(0x7 << 2);
			PORTD = dbits;

			//
			// pixels 3-8 go to IO pins C5-C0
			//
			cbits = ((fb[3+9] & 0x8) << 2) |
					((fb[4+9] & 0x8) << 1) |
					((fb[5+9] & 0x8))      |
					((fb[6+9] & 0x8) >> 1) |
					((fb[7+9] & 0x8) >> 2) |
					((fb[8+9] & 0x8) >> 3);
			PORTC = cbits;

			output_low(PP2);	// finally, enable this plane!
			break;

		case 2:
			//
			// pixels 0,1,2 go to IO pins D7,D6,D5
			//	also, add in 3 bits to turn off all the planes (PP1-PP3) by pulling them high
			//
			dbits = ((fb[0+18] & 0x8) << 4) |
					((fb[1+18] & 0x8) << 3) |
					((fb[2+18] & 0x8) << 2) |
					(0x7 << 2);
			PORTD = dbits;

			//
			// pixels 3-8 go to IO pins C5-C0
			//
			cbits = ((fb[3+18] & 0x8) << 2) |
					((fb[4+18] & 0x8) << 1) |
					((fb[5+18] & 0x8))      |
					((fb[6+18] & 0x8) >> 1) |
					((fb[7+18] & 0x8) >> 2) |
					((fb[8+18] & 0x8) >> 3);
			PORTC = cbits;

			output_low(PP3);	// finally, enable this plane!
			break;

	}

	// finally, increment plane
	_CurPlane++;
	if (_CurPlane >= _NUM_PLANES) {
		_CurPlane = 0;
	}
}

