/*
  Cube_test.pde v1.0  --  Demonstrates how to use the Arduino library for LEDcube V.2 kit

  If the button is not pushed during startup,
     the LEDcube displays the test animation sequence:  each LED blinks once in sequence
  If the button is pushed during startup,
     the LEDcube displays the more interesting animation sequence.


       This work is licensed under the Creative Commons BY SA License.Creative Commons v3.5
            Attribution and Share Alike.
   	see http://creativecommons.org/licenses/by-sa/3.0/ for details.
       Rolf van Widenfelt, Mitch Altman.
 */

#include <avr/pgmspace.h>       // definitions for keeping constants in program memory
#include <LEDcube.h>

// note that wiring.h gets included automatically

// for cube.DrawPixel(c,r,p):
//   c: column
//   r: row
//   p: plane
// where 0,0,0 is the left column, rear row, top plane   --   the board is oriented with the FDTI connector on the left


// create a "cube" instance
LEDcube cube;

/*
// image pixel info and duration will be read from imageTab to these variables in the main Loop
unsigned char Col, Row, Plane;         // col, row plane  (0, 0, 0) is left-rear-top
unsigned long int Duration;            // duration to display a pixel (in millisec) before displaying the next pixel
*/

// index for imageTab
uint16_t index;

// TabOffset will be added to the address of imageTab, and is equal to either 0
//     (so the result is the address of imageTab)
// or the difference between the addresses imageTab2 and imageTab
//     (so the result is the address of imageTab2)
uint16_t TabOffset;


// table of values for 3x3x3 LEDs
// 0 is off, 1 is on for each LED
// last element must have 0 duration
// all row variables only need 3 bits,
//   but for simplicity of programming these each have 8-bit values in this firmware
struct imageElement {
  uint8_t FrontTopRow;      // 3 bits:  left-front-top, mid-front-top, right-front-top
  uint8_t MidTopRow;        // 3 bits:  left-mid-top,   mid-mid-top,   right-mid-top
  uint8_t RearTopRow;       // 3-bits:  left-rear-top,  mid-rear-top,  right-rear-top
  uint8_t FrontMidRow;      // 3-bits:  left-front-mid, mid-front-mid, right-front-mid
  uint8_t MidMidRow;	    // 3 bits:  left-mid-mid,   mid-mid-mid,   right-mid-mid
  uint8_t RearMidRow;	    // 3-bits:  left-rear-mid,  mid-rear-mid,  right-rear-mid
  uint8_t FrontBotRow;      // 3-bits:  left-front-bot, mid-front-bot, right-front-bot
  uint8_t MidBotRow;	    // 3 bits:  left-mid-bot,   mid-mid-bot,   right-mid-bot
  uint8_t RearBotRow;	    // 3-bits:  left-rear-bot,  mid-rear-bot,  right-rear-bot
  uint16_t imageDuration;   // Duration for this element to be displayed before going to next element (divide by 1,000 to get seconds)
} const imageTab[] PROGMEM = {
  { 0b001, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },  // this is a test display sequence that lights up each LED for 1/4 sec, each in turn
  { 0b010, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b100, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b001, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b010, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b100, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b001, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b100, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b001, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b100, 0b000, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b001, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b100, 0b000, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b001, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 0b000, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b001, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 0b000, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b001, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b010, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 0b000, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b001, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b010, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 250 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000,   0 },  // this is a dummy element for end of table (duration=0)
};

const imageElement imageTab2[] PROGMEM = {
  { 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 300 },  // this section is a lit up plane on top that moves to the middle plane, then the bottom plane
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 450 },

  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b111, 0b111, 0b111, 600 },  // this section is a square that grows and shrinks faster and faster, and then slower and slower
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 750 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 750 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 750 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 750 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 600 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 600 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 600 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 600 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 450 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 450 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 450 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 450 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 300 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 150 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 150 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 150 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 150 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 110 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 110 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 110 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 110 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110,  80 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110,  80 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100,  80 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110,  80 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 110 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 110 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 110},
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 110 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 150 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 150 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 150 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 150 },
  { 0b111, 0b111, 0b111, 0b111, 0b101, 0b111, 0b111, 0b111, 0b111, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b100, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b110, 0b110, 0b000, 0b110, 0b110, 300 },
  { 0b111, 0b111, 0b111, 0b111, 0b111, 0b111, 0b111, 0b111, 0b111, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b111, 0b111, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 300 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 300 },

  { 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 300 },  // this section lights up a plane at a time, and goes up and down faster and faster, and then slower and slower
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 300 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 150 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 150 },
  { 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 150 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 150 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 110 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 110 },
  { 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 110 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 110 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 110 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 110 },
  { 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 110 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 110 },
  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 150 },
  { 0b000, 0b000, 0b000, 0b111, 0b111, 0b111, 0b000, 0b000, 0b000, 150 },

  { 0b000, 0b000, 0b000, 0b010, 0b010, 0b010, 0b000, 0b000, 0b000, 130 },  // this section is a line in the middle plane that spins around 4 times
  { 0b000, 0b000, 0b000, 0b001, 0b010, 0b100, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b000, 0b111, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b100, 0b010, 0b001, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b010, 0b010, 0b010, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b001, 0b010, 0b100, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b000, 0b111, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b100, 0b010, 0b001, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b010, 0b010, 0b010, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b001, 0b010, 0b100, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b000, 0b111, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b100, 0b010, 0b001, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b010, 0b010, 0b010, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b001, 0b010, 0b100, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b000, 0b111, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b000, 0b100, 0b010, 0b001, 0b000, 0b000, 0b000, 130 },

  { 0b001, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b100, 130 },  // this section is a diagnal line that spins around 4 times
  { 0b000, 0b000, 0b001, 0b000, 0b010, 0b000, 0b100, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b100, 0b000, 0b010, 0b000, 0b001, 0b000, 0b000, 130 },
  { 0b100, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b001, 130 },
  { 0b001, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b100, 130 },
  { 0b000, 0b000, 0b001, 0b000, 0b010, 0b000, 0b100, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b100, 0b000, 0b010, 0b000, 0b001, 0b000, 0b000, 130 },
  { 0b100, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b001, 130 },
  { 0b001, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b100, 130 },
  { 0b000, 0b000, 0b001, 0b000, 0b010, 0b000, 0b100, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b100, 0b000, 0b010, 0b000, 0b001, 0b000, 0b000, 130 },
  { 0b100, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b001, 130 },
  { 0b001, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b100, 130 },
  { 0b000, 0b000, 0b001, 0b000, 0b010, 0b000, 0b100, 0b000, 0b000, 130 },
  { 0b000, 0b000, 0b100, 0b000, 0b010, 0b000, 0b001, 0b000, 0b000, 130 },
  { 0b100, 0b000, 0b000, 0b000, 0b010, 0b000, 0b000, 0b000, 0b001, 130 },

  { 0b010, 0b010, 0b010, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },  // this section is a line on the top plane that spins around 3 times
  { 0b001, 0b010, 0b100, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b100, 0b010, 0b001, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b010, 0b010, 0b010, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b001, 0b010, 0b100, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b100, 0b010, 0b001, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b010, 0b010, 0b010, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b001, 0b010, 0b100, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b000, 0b111, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },
  { 0b100, 0b010, 0b001, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 130 },

  { 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000, 0b000,   0 },  // this is a dummy element for end of table (duration=0)
};


// This function displays a 3x3x3 image by reading an element from ImageTab, and keeping displayed for the duration specified in the element
void displayImage(int i) {
  uint16_t duration = pgm_read_word( &(imageTab+TabOffset)[i].imageDuration );
  uint8_t tempA, tempB, tempC;

// turn on all specified pixels in Top plane

tempA = pgm_read_byte( &(imageTab+TabOffset)[index].FrontTopRow );  // we now have 3 bits: left-front-top, mid-front-top, right-front-top
  tempB = tempA & 0b010;   // tempB is now high if mid-front-top LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-front-top LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-front-top LED should be turned on
  if ( tempA ) cube.DrawPixel(2,2,0);  // turn on right-front-top LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,2,0);  // turn on mid-front-top LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,2,0);  // turn on left-front-top LED if it should be turned on

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].MidTopRow );  // we now have 3 bits: left-mid-top,   mid-mid-top,   right-mid-top
  tempB = tempA & 0b010;   // tempB is now high if mid-mid-top LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-mid-top LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-mid-top LED should be turned on
  if ( tempA ) cube.DrawPixel(2,1,0);  // turn on right-mid-top LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,1,0);  // turn on mid-mid-top LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,1,0);  // turn on left-mid-top LED if it should be turned on

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].RearTopRow );  // we now have 3 bits: left-rear-top,   mid-rear-top,   right-rear-top
  tempB = tempA & 0b010;   // tempB is now high if mid-rear-top LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-rear-top LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-rear-top LED should be turned on
  if ( tempA ) cube.DrawPixel(2,0,0);  // turn on right-rear-top LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,0,0);  // turn on mid-rear-top LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,0,0);  // turn on left-rear-top LED if it should be turned on

// turn on all specified pixels in plane

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].FrontMidRow );  // we now have 3 bits: left-front-mid, mid-front-mid, right-front-mid
  tempB = tempA & 0b010;   // tempB is now high if mid-front-mid LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-front-mid LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-front-mid LED should be turned on
  if ( tempA ) cube.DrawPixel(2,2,1);  // turn on right-front-mid LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,2,1);  // turn on mid-front-mid LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,2,1);  // turn on left-front-mid LED if it should be turned on

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].MidMidRow );  // we now have 3 bits: left-mid-mid,   mid-mid-mid,   right-mid-mid
  tempB = tempA & 0b010;   // tempB is now high if mid-mid-mid LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-mid-mid LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-mid-mid LED should be turned on
  if ( tempA ) cube.DrawPixel(2,1,1);  // turn on right-mid-mid LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,1,1);  // turn on mid-mid-mid LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,1,1);  // turn on left-mid-mid LED if it should be turned on

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].RearMidRow );  // we now have 3 bits: left-rear-mid,   mid-rear-mid,   right-rear-mid
  tempB = tempA & 0b010;   // tempB is now high if mid-rear-mid LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-rear-mid LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-rear-mid LED should be turned on
  if ( tempA ) cube.DrawPixel(2,0,1);  // turn on right-rear-mid LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,0,1);  // turn on mid-rear-mid LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,0,1);  // turn on left-rear-mid LED if it should be turned on

// turn on all specified pixels in Bottom plane

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].FrontBotRow );  // we now have 3 bits: left-front-bot, mid-front-bot, right-front-bot
  tempB = tempA & 0b010;   // tempB is now high if mid-front-bot LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-front-bot LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-front-bot LED should be turned on
  if ( tempA ) cube.DrawPixel(2,2,2);  // turn on right-front-bot LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,2,2);  // turn on mid-front-bot LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,2,2);  // turn on left-front-bot LED if it should be turned on

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].MidBotRow );  // we now have 3 bits: left-mid-bot,   mid-mid-bot,   right-mid-bot
  tempB = tempA & 0b010;   // tempB is now high if mid-mid-bot LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-mid-bot LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-mid-bot LED should be turned on
  if ( tempA ) cube.DrawPixel(2,1,2);  // turn on right-mid-bot LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,1,2);  // turn on mid-mid-bot LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,1,2);  // turn on left-mid-bot LED if it should be turned on

  tempA = pgm_read_byte( &(imageTab+TabOffset)[index].RearBotRow );  // we now have 3 bits: left-rear-bot,   mid-rear-bot,   right-rear-bot
  tempB = tempA & 0b010;   // tempB is now high if mid-rear-bot LED should be turned on
  tempC = tempA & 0b100;   // tempC is now high if left-rear-bot LED should be turned on
  tempA = tempA & 0b001;   // tempA is now high if right-rear-bot LED should be turned on
  if ( tempA ) cube.DrawPixel(2,0,2);  // turn on right-rear-bot LED if it should be turned on
  if ( tempB ) cube.DrawPixel(1,0,2);  // turn on mid-rear-bot LED if it should be turned on
  if ( tempC ) cube.DrawPixel(0,0,2);  // turn on left-rear-bot LED if it should be turned on

  // keep displaying these LEDs for specified duration
  delay(duration);
}


void setup() {
  // first, initialize the library - this is needed!
  cube.Begin();

  // if the button is not being pushed then TabOffset = 0, so that the address (imageTab + TabOffset)
  //     is the address of imageTab, the test image, so we display the test animation sequence
  // else TabOffset = the difference between imageTab2 and imageTab, so that the address (imageTab + TabOffset)
  //     is the address of imageTab2, the more interesting imageTab2, to display the more interesting animation sequence
  if ( cube.GetButton(BUTTON_A) ) {
    TabOffset = imageTab2 - imageTab;
  }
  else {
    TabOffset = 0;
  }

  // initially, we want all the pixels off
  cube.Clear();
}


void loop() {

  // loop through and display entire Image Table of Image Elements
  index = 0;
  while (pgm_read_word(&(imageTab+TabOffset)[index].imageDuration) != 0) {  // 0 signifies end of table
    displayImage(index);
    cube.Clear();
    index++;
  }
}
